@@ -11,26 +11,30 @@ from uuid import uuid4
 
 import httpx
 import jwt
-from Crypto.Cipher import AES, PKCS1_v1_5
+from Crypto.Cipher import PKCS1_v1_5
 from Crypto.PublicKey import RSA
-from Crypto.Util.Padding import pad
 
-from bimmer_connected.api.regions import Regions, get_aes_keys, get_app_version, get_ocp_apim_key, get_server_url
+from bimmer_connected.api.regions import Regions, get_app_version, get_ocp_apim_key, get_server_url, get_user_agent
 from bimmer_connected.api.utils import (
     create_s256_code_challenge,
+    generate_cn_nonce,
     generate_token,
+    get_capture_position,
     get_correlation_id,
-    handle_http_status_error,
+    handle_httpstatuserror,
+    try_import_pillow_image,
 )
 from bimmer_connected.const import (
+    AUTH_CHINA_CAPTCHA_CHECK_URL,
+    AUTH_CHINA_CAPTCHA_URL,
     AUTH_CHINA_LOGIN_URL,
     AUTH_CHINA_PUBLIC_KEY_URL,
     AUTH_CHINA_TOKEN_URL,
     HTTPX_TIMEOUT,
     OAUTH_CONFIG_URL,
-    USER_AGENT,
     X_USER_AGENT,
 )
+from bimmer_connected.models import MyBMWAPIError
 
 EXPIRES_AT_OFFSET = datetime.timedelta(seconds=HTTPX_TIMEOUT * 2)
 
@@ -38,9 +42,8 @@ _LOGGER = logging.getLogger(__name__)
 
 
 class MyBMWAuthentication(httpx.Auth):
-    """Authentication for MyBMW API."""
+    """Authentication and Retry Handler for MyBMW API."""
 
-    # pylint: disable=too-many-arguments,too-many-instance-attributes
     def __init__(
         self,
         username: str,
@@ -49,6 +52,7 @@ class MyBMWAuthentication(httpx.Auth):
         access_token: Optional[str] = None,
         expires_at: Optional[datetime.datetime] = None,
         refresh_token: Optional[str] = None,
+        gcid: Optional[str] = None,
     ):
         self.username: str = username
         self.password: str = password
@@ -58,10 +62,11 @@ class MyBMWAuthentication(httpx.Auth):
         self.refresh_token: Optional[str] = refresh_token
         self.session_id: str = str(uuid4())
         self._lock: Optional[asyncio.Lock] = None
+        self.gcid: Optional[str] = gcid
 
     @property
     def login_lock(self) -> asyncio.Lock:
-        """Makes sure that there is a lock in the current event loop."""
+        """Make sure that there is a lock in the current event loop."""
         if not self._lock:
             self._lock = asyncio.Lock()
         return self._lock
@@ -79,7 +84,9 @@ class MyBMWAuthentication(httpx.Auth):
 
         # Try getting a response
         response: httpx.Response = (yield request)
+        await response.aread()
 
+        # Handle "classic" 401 Unauthorized
         if response.status_code == 401:
             async with self.login_lock:
                 _LOGGER.debug("Received unauthorized response, refreshing token.")
@@ -87,7 +94,8 @@ class MyBMWAuthentication(httpx.Auth):
             request.headers["authorization"] = f"Bearer {self.access_token}"
             request.headers["bmw-session-id"] = self.session_id
             yield request
-        elif response.status_code == 429:
+        # Quota errors can either be 429 Too Many Requests or 403 Quota Exceeded (instead of 403 Forbidden)
+        elif response.status_code == 429 or (response.status_code == 403 and "quota" in response.text.lower()):
             for _ in range(3):
                 if response.status_code == 429:
                     await response.aread()
@@ -98,7 +106,10 @@ class MyBMWAuthentication(httpx.Auth):
                     await asyncio.sleep(wait_time)
                     response = yield request
             # Raise if still error after 3rd retry
-            response.raise_for_status()
+            try:
+                response.raise_for_status()
+            except httpx.HTTPStatusError as ex:
+                await handle_httpstatuserror(ex, log_handler=_LOGGER)
 
     async def login(self) -> None:
         """Get a valid OAuth token."""
@@ -118,87 +129,90 @@ class MyBMWAuthentication(httpx.Auth):
             if not token_data:
                 token_data = await self._login_china()
             token_data["expires_at"] = token_data["expires_at"] - EXPIRES_AT_OFFSET
+            self.gcid = token_data["gcid"]
 
         self.access_token = token_data["access_token"]
         self.expires_at = token_data["expires_at"]
         self.refresh_token = token_data["refresh_token"]
 
-    async def _login_row_na(self):  # pylint: disable=too-many-locals
+    async def _login_row_na(self):
         """Login to Rest of World and North America."""
-        try:
-            async with MyBMWLoginClient(region=self.region) as client:
-                _LOGGER.debug("Authenticating with MyBMW flow for North America & Rest of World.")
+        async with MyBMWLoginClient(region=self.region) as client:
+            _LOGGER.debug("Authenticating with MyBMW flow for North America & Rest of World.")
 
-                # Get OAuth2 settings from BMW API
-                r_oauth_settings = await client.get(
-                    OAUTH_CONFIG_URL,
-                    headers={
-                        "ocp-apim-subscription-key": get_ocp_apim_key(self.region),
-                        "bmw-session-id": self.session_id,
-                        **get_correlation_id(),
-                    },
-                )
-                oauth_settings = r_oauth_settings.json()
+            # Get OAuth2 settings from BMW API
+            r_oauth_settings = await client.get(
+                OAUTH_CONFIG_URL,
+                headers={
+                    "ocp-apim-subscription-key": get_ocp_apim_key(self.region),
+                    "bmw-session-id": self.session_id,
+                    **get_correlation_id(),
+                },
+            )
+            oauth_settings = r_oauth_settings.json()
 
-                # Generate OAuth2 Code Challenge + State
-                code_verifier = generate_token(86)
-                code_challenge = create_s256_code_challenge(code_verifier)
+            # Generate OAuth2 Code Challenge + State
+            code_verifier = generate_token(86)
+            code_challenge = create_s256_code_challenge(code_verifier)
 
-                state = generate_token(22)
+            state = generate_token(22)
 
-                # Set up authenticate endpoint
-                authenticate_url = oauth_settings["tokenEndpoint"].replace("/token", "/authenticate")
-                oauth_base_values = {
-                    "client_id": oauth_settings["clientId"],
-                    "response_type": "code",
-                    "redirect_uri": oauth_settings["returnUrl"],
-                    "state": state,
-                    "nonce": "login_nonce",
-                    "scope": " ".join(oauth_settings["scopes"]),
-                    "code_challenge": code_challenge,
-                    "code_challenge_method": "S256",
-                }
+            # Set up authenticate endpoint
+            authenticate_url = oauth_settings["tokenEndpoint"].replace("/token", "/authenticate")
+            oauth_base_values = {
+                "client_id": oauth_settings["clientId"],
+                "response_type": "code",
+                "redirect_uri": oauth_settings["returnUrl"],
+                "state": state,
+                "nonce": "login_nonce",
+                "scope": " ".join(oauth_settings["scopes"]),
+                "code_challenge": code_challenge,
+                "code_challenge_method": "S256",
+            }
 
-                # Call authenticate endpoint first time (with user/pw) and get authentication
-                response = await client.post(
-                    authenticate_url,
-                    data=dict(
-                        oauth_base_values,
-                        **{
-                            "grant_type": "authorization_code",
-                            "username": self.username,
-                            "password": self.password,
-                        },
-                    ),
-                )
-                authorization = httpx.URL(response.json()["redirect_to"]).params["authorization"]
-
-                # With authorization, call authenticate endpoint second time to get code
-                response = await client.post(
-                    authenticate_url,
-                    data=dict(oauth_base_values, **{"authorization": authorization}),
-                )
-                code = response.next_request.url.params["code"]
-
-                # With code, get token
-                current_utc_time = datetime.datetime.utcnow()
-                response = await client.post(
-                    oauth_settings["tokenEndpoint"],
-                    data={
-                        "code": code,
-                        "code_verifier": code_verifier,
-                        "redirect_uri": oauth_settings["returnUrl"],
+            # Call authenticate endpoint first time (with user/pw) and get authentication
+            response = await client.post(
+                authenticate_url,
+                data=dict(
+                    oauth_base_values,
+                    **{
                         "grant_type": "authorization_code",
+                        "username": self.username,
+                        "password": self.password,
                     },
-                    auth=(oauth_settings["clientId"], oauth_settings["clientSecret"]),
-                )
-                response_json = response.json()
+                ),
+            )
+            authorization = httpx.URL(response.json()["redirect_to"]).params["authorization"]
 
-                expiration_time = int(response_json["expires_in"])
-                expires_at = current_utc_time + datetime.timedelta(seconds=expiration_time)
+            # With authorization, call authenticate endpoint second time to get code
+            response = await client.post(
+                authenticate_url,
+                params={
+                    "interaction-id": uuid4(),
+                    "client-version": X_USER_AGENT.format(
+                        brand="bmw", app_version=get_app_version(self.region), region=self.region.value
+                    ),
+                },
+                data=dict(oauth_base_values, **{"authorization": authorization}),
+            )
+            code = response.next_request.url.params["code"]
 
-        except httpx.HTTPStatusError as ex:
-            handle_http_status_error(ex, "Authentication", _LOGGER)
+            # With code, get token
+            current_utc_time = datetime.datetime.now(tz=datetime.timezone.utc)
+            response = await client.post(
+                oauth_settings["tokenEndpoint"],
+                data={
+                    "code": code,
+                    "code_verifier": code_verifier,
+                    "redirect_uri": oauth_settings["returnUrl"],
+                    "grant_type": "authorization_code",
+                },
+                auth=(oauth_settings["clientId"], oauth_settings["clientSecret"]),
+            )
+            response_json = response.json()
+
+            expiration_time = int(response_json["expires_in"])
+            expires_at = current_utc_time + datetime.timedelta(seconds=expiration_time)
 
         return {
             "access_token": response_json["access_token"],
@@ -224,7 +238,7 @@ class MyBMWAuthentication(httpx.Auth):
                 oauth_settings = r_oauth_settings.json()
 
                 # With code, get token
-                current_utc_time = datetime.datetime.utcnow()
+                current_utc_time = datetime.datetime.now(tz=datetime.timezone.utc)
                 response = await client.post(
                     oauth_settings["tokenEndpoint"],
                     data={
@@ -240,9 +254,8 @@ class MyBMWAuthentication(httpx.Auth):
                 expiration_time = int(response_json["expires_in"])
                 expires_at = current_utc_time + datetime.timedelta(seconds=expiration_time)
 
-        except httpx.HTTPStatusError as ex:
-            _LOGGER.debug("Unable to get access token using refresh token.")
-            handle_http_status_error(ex, "Authentication", _LOGGER, debug=True)
+        except MyBMWAPIError:
+            _LOGGER.debug("Unable to get access token using refresh token, falling back to username/password.")
             return {}
 
         return {
@@ -252,43 +265,55 @@ class MyBMWAuthentication(httpx.Auth):
         }
 
     async def _login_china(self):
-        try:
-            async with MyBMWLoginClient(region=self.region) as client:
-                _LOGGER.debug("Authenticating with MyBMW flow for China.")
+        async with MyBMWLoginClient(region=self.region) as client:
+            _LOGGER.debug("Authenticating with MyBMW flow for China.")
 
-                # Get current RSA public certificate & use it to encrypt password
-                response = await client.get(
-                    AUTH_CHINA_PUBLIC_KEY_URL,
-                )
-                pem_public_key = response.json()["data"]["value"]
+            # While PIL.Image is only needed in `get_capture_position`, we test it here to avoid
+            # unneeded requests to the server.
+            try_import_pillow_image()
 
-                public_key = RSA.import_key(pem_public_key)
-                cipher_rsa = PKCS1_v1_5.new(public_key)
-                encrypted = cipher_rsa.encrypt(self.password.encode())
-                pw_encrypted = base64.b64encode(encrypted).decode("UTF-8")
+            # Get current RSA public certificate & use it to encrypt password
+            response = await client.get(
+                AUTH_CHINA_PUBLIC_KEY_URL,
+            )
+            pem_public_key = response.json()["data"]["value"]
 
-                cipher_aes = AES.new(**get_aes_keys(self.region), mode=AES.MODE_CBC)
-                nonce = f"{self.username}|{datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S.%fZ')}".encode()
+            public_key = RSA.import_key(pem_public_key)
+            cipher_rsa = PKCS1_v1_5.new(public_key)
+            encrypted = cipher_rsa.encrypt(self.password.encode())
+            pw_encrypted = base64.b64encode(encrypted).decode("UTF-8")
 
-                # Get token
-                response = await client.post(
-                    AUTH_CHINA_LOGIN_URL,
-                    headers={"x-login-nonce": base64.b64encode(cipher_aes.encrypt(pad(nonce, 16))).decode()},
-                    json={"mobile": self.username, "password": pw_encrypted},
-                )
-                response_json = response.json()["data"]
+            captcha_res = await client.post(
+                AUTH_CHINA_CAPTCHA_URL,
+                json={"mobile": self.username},
+            )
+            verify_id = captcha_res.json()["data"]["verifyId"]
 
-                decoded_token = jwt.decode(
-                    response_json["access_token"], algorithms=["HS256"], options={"verify_signature": False}
-                )
+            position = get_capture_position(captcha_res.json()["data"]["backGroundImg"])
+            await client.post(AUTH_CHINA_CAPTCHA_CHECK_URL, json={"position": position, "verifyId": verify_id})
 
-        except httpx.HTTPStatusError as ex:
-            handle_http_status_error(ex, "Authentication", _LOGGER)
+            # Get token
+            response = await client.post(
+                AUTH_CHINA_LOGIN_URL,
+                headers={"x-login-nonce": generate_cn_nonce(self.username)},
+                json={
+                    "mobile": self.username,
+                    "password": pw_encrypted,
+                    "verifyId": verify_id,
+                    "deviceId": self.username,
+                },
+            )
+            response_json = response.json()["data"]
+
+            decoded_token = jwt.decode(
+                response_json["access_token"], algorithms=["HS256"], options={"verify_signature": False}
+            )
 
         return {
             "access_token": response_json["access_token"],
-            "expires_at": datetime.datetime.utcfromtimestamp(decoded_token["exp"]),
+            "expires_at": datetime.datetime.fromtimestamp(decoded_token["exp"], tz=datetime.timezone.utc),
             "refresh_token": response_json["refresh_token"],
+            "gcid": response_json["gcid"],
         }
 
     async def _refresh_token_china(self):
@@ -296,11 +321,12 @@ class MyBMWAuthentication(httpx.Auth):
             async with MyBMWLoginClient(region=self.region) as client:
                 _LOGGER.debug("Authenticating with refresh token for China.")
 
-                current_utc_time = datetime.datetime.utcnow()
+                current_utc_time = datetime.datetime.now(tz=datetime.timezone.utc)
 
                 # Try logging in using refresh_token
                 response = await client.post(
                     AUTH_CHINA_TOKEN_URL,
+                    headers={"x-login-nonce": generate_cn_nonce(self.gcid)},
                     data={
                         "refresh_token": self.refresh_token,
                         "grant_type": "refresh_token",
@@ -311,15 +337,15 @@ class MyBMWAuthentication(httpx.Auth):
                 expiration_time = int(response_json["expires_in"])
                 expires_at = current_utc_time + datetime.timedelta(seconds=expiration_time)
 
-        except httpx.HTTPStatusError as ex:
-            _LOGGER.debug("Unable to get access token using refresh token.")
-            handle_http_status_error(ex, "Authentication", _LOGGER, debug=True)
+        except MyBMWAPIError:
+            _LOGGER.debug("Unable to get access token using refresh token, falling back to username/password.")
             return {}
 
         return {
             "access_token": response_json["access_token"],
             "expires_at": expires_at,
             "refresh_token": response_json["refresh_token"],
+            "gcid": response_json["gcid"],
         }
 
 
@@ -336,7 +362,7 @@ class MyBMWLoginClient(httpx.AsyncClient):
         region = kwargs.pop("region")
         kwargs["base_url"] = get_server_url(region)
         kwargs["headers"] = {
-            "user-agent": USER_AGENT,
+            "user-agent": get_user_agent(region),
             "x-user-agent": X_USER_AGENT.format(brand="bmw", app_version=get_app_version(region), region=region.value),
         }
 
@@ -347,11 +373,13 @@ class MyBMWLoginClient(httpx.AsyncClient):
         async def raise_for_status_event_handler(response: httpx.Response):
             """Event handler that automatically raises HTTPStatusErrors when attached.
 
-            Will only raise on 4xx/5xx errors (but not on 429) and not raise on 3xx.
+            Will only raise on 4xx/5xx errors but not 429 which is handled `self.auth`.
             """
-            if response.is_error and not response.status_code == 429:
-                await response.aread()
-                response.raise_for_status()
+            if response.is_error and response.status_code != 429:
+                try:
+                    response.raise_for_status()
+                except httpx.HTTPStatusError as ex:
+                    await handle_httpstatuserror(ex, log_handler=_LOGGER)
 
         kwargs["event_hooks"]["response"].append(raise_for_status_event_handler)
 
@@ -377,6 +405,10 @@ class MyBMWLoginRetry(httpx.Auth):
                 _LOGGER.debug("Sleeping %s seconds due to 429 Too Many Requests", wait_time)
                 await asyncio.sleep(wait_time)
                 response = yield request
+        # Only checking for 429 errors, as all other errors are handled by the
+        # response hook of MyBMWLoginClient
         if response.status_code == 429:
-            await response.aread()
-            response.raise_for_status()
+            try:
+                response.raise_for_status()
+            except httpx.HTTPStatusError as ex:
+                await handle_httpstatuserror(ex, log_handler=_LOGGER)
