@@ -1,8 +1,8 @@
 """Get the right url for the different countries."""
 from base64 import b64decode
-from typing import Dict, List
+from typing import List
 
-from bimmer_connected.const import AES_KEYS, APP_VERSIONS, OCP_APIM_KEYS, SERVER_URLS_MYBMW, Regions
+from bimmer_connected.const import APP_VERSIONS, OCP_APIM_KEYS, SERVER_URLS_MYBMW, USER_AGENTS, Regions
 
 
 def valid_regions() -> List[str]:
@@ -26,6 +26,11 @@ def get_server_url(region: Regions) -> str:
     return f"https://{SERVER_URLS_MYBMW[region]}"
 
 
+def get_user_agent(region: Regions) -> str:
+    """Get the Dart user agent for the region."""
+    return USER_AGENTS[region]
+
+
 def get_app_version(region: Regions) -> str:
     """Get the app version & build number for the region."""
     return APP_VERSIONS[region]
@@ -34,8 +39,3 @@ def get_app_version(region: Regions) -> str:
 def get_ocp_apim_key(region: Regions) -> str:
     """Get the authorization for OAuth settings."""
     return b64decode(OCP_APIM_KEYS[region]).decode()
-
-
-def get_aes_keys(region: Regions) -> Dict[str, bytes]:
-    """Get the keys for login nonce."""
-    return {k: b64decode(v) for k, v in AES_KEYS[region].items()}
