@@ -5,6 +5,8 @@ from dataclasses import InitVar, dataclass, field
 from enum import Enum
 from typing import Dict, List, NamedTuple, Optional, Tuple, Union
 
+from bimmer_connected.const import DEFAULT_POI_NAME
+
 _LOGGER = logging.getLogger(__name__)
 
 
@@ -31,14 +33,14 @@ class VehicleDataBase:
 
     @classmethod
     def from_vehicle_data(cls, vehicle_data: Dict):
-        """Creates the class based on vehicle data from API."""
+        """Create the class based on vehicle data from API."""
         parsed = cls._parse_vehicle_data(vehicle_data) or {}
         if len(parsed) > 0:
             return cls(**parsed)
         return None
 
     def update_from_vehicle_data(self, vehicle_data: Dict):
-        """Updates the attributes based on vehicle data from API."""
+        """Update the attributes based on vehicle data from API."""
         parsed = self._parse_vehicle_data(vehicle_data) or {}
         parsed.update(self._update_after_parse(parsed))
         if len(parsed) > 0:
@@ -46,11 +48,11 @@ class VehicleDataBase:
 
     @classmethod
     def _parse_vehicle_data(cls, vehicle_data: Dict) -> Optional[Dict]:
-        """Parses desired attributes out of vehicle data from API."""
+        """Parse desired attributes out of vehicle data from API."""
         raise NotImplementedError()
 
     def _update_after_parse(self, parsed: Dict) -> Dict:
-        """Updates parsed vehicle data with attributes stored in class if needed."""
+        """Update parsed vehicle data with attributes stored in class if needed."""
         return parsed
 
 
@@ -62,11 +64,9 @@ class GPSPosition:
     longitude: Optional[float]
 
     def __post_init__(self):
-        # pylint: disable=no-member
         if len([v for v in self.__dict__.values() if v is None]) not in [0, len(self.__dataclass_fields__)]:
             raise TypeError("Either none or all arguments must be 'None'.")
 
-        # pylint: disable=no-member
         for field_name in self.__dataclass_fields__:
             value = getattr(self, field_name)
             if value is not None and not isinstance(value, (float, int)):
@@ -93,10 +93,21 @@ class PointOfInterestAddress:
     """Address data of a PointOfInterest."""
 
     street: Optional[str] = None
-    postalCode: Optional[str] = None  # pylint: disable=invalid-name
+    postalCode: Optional[str] = None
     city: Optional[str] = None
     country: Optional[str] = None
 
+    # The following attributes are not by us but available in the API
+    banchi: Optional[str] = None
+    chome: Optional[str] = None
+    countryCode: Optional[str] = None
+    district: Optional[str] = None
+    go: Optional[str] = None
+    houseNumber: Optional[str] = None
+    region: Optional[str] = None
+    regionCode: Optional[str] = None
+    settlement: Optional[str] = None
+
 
 @dataclass
 class PointOfInterest:
@@ -104,21 +115,37 @@ class PointOfInterest:
 
     lat: InitVar[float]
     lon: InitVar[float]
-    name: Optional[str] = None
+    name: Optional[str] = DEFAULT_POI_NAME
     street: InitVar[str] = None
     postal_code: InitVar[str] = None
     city: InitVar[str] = None
     country: InitVar[str] = None
 
     coordinates: GPSPosition = field(init=False)
-    locationAddress: Optional[PointOfInterestAddress] = field(init=False)  # pylint: disable=invalid-name
-    entryPoints: List = field(init=False, default_factory=list)  # pylint: disable=invalid-name
+    locationAddress: Optional[PointOfInterestAddress] = field(init=False)
+    # The following attributes are not by us but required in the API
+    formattedAddress: Optional[str] = None
+    entryPoints: List = field(init=False, default_factory=list)
 
-    def __post_init__(self, lat, lon, street, postal_code, city, country):  # pylint: disable=too-many-arguments
+    # The following attributes are not by us but available in the API
+    address: Optional[str] = None
+    baseCategoryId: Optional[str] = None
+    phoneNumber: Optional[str] = None
+    provider: Optional[str] = None
+    providerId: Optional[str] = None
+    providerPoiId: str = ""
+    sourceType: Optional[str] = None
+    type: Optional[str] = None
+    vehicleCategoryId: Optional[str] = None
+
+    def __post_init__(self, lat, lon, street, postal_code, city, country):
         self.coordinates = GPSPosition(lat, lon)
-        # pylint: disable=invalid-name
+
         self.locationAddress = PointOfInterestAddress(street, postal_code, city, country)
 
+        if not self.formattedAddress:
+            self.formattedAddress = ", ".join([i for i in [street, postal_code, city] if i]) or "Coordinates only"
+
 
 class ValueWithUnit(NamedTuple):
     """A value with a corresponding unit."""
@@ -133,3 +160,29 @@ class AnonymizedResponse:
 
     filename: str
     content: Optional[Union[List, Dict, str]] = None
+
+
+@dataclass
+class ChargingSettings:
+    """Charging settings to control the vehicle."""
+
+    chargingTarget: Optional[int]
+    isUnlockCableActive = None
+    acLimitValue: Optional[int] = None
+    dcLoudness = None
+
+
+class MyBMWAPIError(Exception):
+    """General BMW API error."""
+
+
+class MyBMWAuthError(MyBMWAPIError):
+    """Auth-related error from BMW API (HTTP status codes 401 and 403)."""
+
+
+class MyBMWQuotaError(MyBMWAPIError):
+    """Quota exceeded on BMW API."""
+
+
+class MyBMWRemoteServiceError(MyBMWAPIError):
+    """Error when executing remote services."""
