@@ -1,19 +1,19 @@
 """General utils and base classes used in the library."""
 
+
 import datetime
 import inspect
 import json
 import logging
 import pathlib
 import time
-import traceback
 from enum import Enum
-from typing import TYPE_CHECKING, Dict, List, Optional
+from typing import TYPE_CHECKING, Dict, List, Optional, Union
 
 from bimmer_connected.models import AnonymizedResponse
 
 if TYPE_CHECKING:
-    from typing import Callable, TypeVar
+    from typing import TypeVar
 
     from typing_extensions import ParamSpec
 
@@ -24,16 +24,10 @@ _LOGGER = logging.getLogger(__name__)
 
 
 JSON_IGNORED_KEYS = ["account", "_account", "vehicle", "_vehicle", "status", "remote_services"]
-JSON_DEPRECATED_KEYS = [
-    "has_hv_battery",
-    "has_range_extender",
-    "has_internal_combustion_engine",
-    "has_weekly_planner_service",
-]
 
 
 def get_class_property_names(obj: object):
-    """Returns the names of all properties of a class."""
+    """Return the names of all properties of a class."""
     return [p[0] for p in inspect.getmembers(type(obj), inspect.isdatadescriptor) if not p[0].startswith("_")]
 
 
@@ -61,39 +55,18 @@ def parse_datetime(date_str: str) -> Optional[datetime.datetime]:
 class MyBMWJSONEncoder(json.JSONEncoder):
     """JSON Encoder that handles data classes, properties and additional data types."""
 
-    def default(self, o):
+    def default(self, o) -> Union[str, dict]:  # noqa: D102
         if isinstance(o, (datetime.datetime, datetime.date, datetime.time)):
             return o.isoformat()
         if not isinstance(o, Enum) and hasattr(o, "__dict__") and isinstance(o.__dict__, Dict):
             retval: Dict = o.__dict__
-            retval.update({p: getattr(o, p) for p in get_class_property_names(o) if p not in JSON_DEPRECATED_KEYS})
+            retval.update({p: getattr(o, p) for p in get_class_property_names(o)})
             return {k: v for k, v in retval.items() if k not in JSON_IGNORED_KEYS}
         return str(o)
 
 
-def deprecated(replacement: Optional[str] = None) -> "Callable[[Callable[_P, _R]], Callable[_P, _R | None]]":
-    """Mark a function or property as deprecated."""
-
-    def decorator(func: "Callable[_P, _R]") -> "Callable[_P, _R | None]":
-        def _func_wrapper(*args: "_P.args", **kwargs: "_P.kwargs") -> "_R | None":
-            replacement_text = f" Please change to '{replacement}'." if replacement else ""
-            stack = traceback.extract_stack()[-2]
-            _LOGGER.warning(
-                "DeprecationWarning:%s:%s: '%s' is deprecated.%s",
-                stack.filename,
-                stack.lineno,
-                func.__qualname__,
-                replacement_text,
-            )
-            return func(*args, **kwargs)
-
-        return _func_wrapper
-
-    return decorator
-
-
 def to_camel_case(input_str: str) -> str:
-    """Converts SNAKE_CASE or snake_case to camelCase."""
+    """Convert SNAKE_CASE or snake_case to camelCase."""
 
     retval = ""
     flag_upper = False
@@ -118,4 +91,4 @@ def log_response_store_to_file(response_store: List[AnonymizedResponse], logfile
             if output_path.suffix == ".json" or not isinstance(content, str):
                 json.dump(content or [], logfile, indent=4, sort_keys=True)
             else:
-                logfile.write((content or "NO CONTENT"))
+                logfile.write(content or "NO CONTENT")
