@@ -6,7 +6,7 @@ import logging
 from dataclasses import dataclass
 from typing import Any, Dict, Optional
 
-from bimmer_connected.const import ATTR_ATTRIBUTES
+from bimmer_connected.const import ATTR_ATTRIBUTES, ATTR_STATE
 from bimmer_connected.models import StrEnum, ValueWithUnit, VehicleDataBase
 from bimmer_connected.vehicle.const import COMBUSTION_ENGINE_DRIVE_TRAINS, HV_BATTERY_DRIVE_TRAINS, DriveTrainType
 
@@ -32,7 +32,7 @@ class ChargingState(StrEnum):
 
 
 @dataclass
-class FuelAndBattery(VehicleDataBase):  # pylint:disable=too-many-instance-attributes
+class FuelAndBattery(VehicleDataBase):
     """Provides an accessible version of `status.FuelAndBattery`."""
 
     remaining_range_fuel: Optional[ValueWithUnit] = ValueWithUnit(None, None)
@@ -65,6 +65,9 @@ class FuelAndBattery(VehicleDataBase):  # pylint:disable=too-many-instance-attri
     is_charger_connected: bool = False
     """Get status of the connection"""
 
+    charging_target: Optional[int] = None
+    """State of charging target in percent."""
+
     account_timezone: datetime.timezone = datetime.timezone.utc
 
     @property
@@ -74,10 +77,9 @@ class FuelAndBattery(VehicleDataBase):  # pylint:disable=too-many-instance-attri
             return self.charging_start_time_no_tz.astimezone(self.account_timezone)
         return None
 
-    # pylint:disable=arguments-differ
     @classmethod
     def from_vehicle_data(cls, vehicle_data: Dict):
-        """Creates the class based on vehicle data from API."""
+        """Create the class based on vehicle data from API."""
         parsed = cls._parse_vehicle_data(vehicle_data) or {}
         if len(parsed) > 0:
             return cls(**parsed)
@@ -89,10 +91,17 @@ class FuelAndBattery(VehicleDataBase):  # pylint:disable=too-many-instance-attri
         retval: Dict[str, Any] = {}
         drivetrain = DriveTrainType(vehicle_data.get(ATTR_ATTRIBUTES, {}).get("driveTrain") or DriveTrainType.UNKNOWN)
 
-        state = vehicle_data.get("state", {})
+        # return early if state data hasn't been loaded or no combustion/electric data is available
+        if (
+            ATTR_STATE not in vehicle_data
+            or len({"combustionFuelLevel", "electricChargingState"}.intersection(set(vehicle_data[ATTR_STATE]))) == 0
+        ):
+            return retval
+
+        state = vehicle_data["state"]
 
         if drivetrain in COMBUSTION_ENGINE_DRIVE_TRAINS:
-            retval.update(cls._parse_fuel_data(state.get("combustionFuelLevel", {}), vehicle_data["is_metric"]))
+            retval.update(cls._parse_fuel_data(state.get("combustionFuelLevel", {})))
 
         if drivetrain in HV_BATTERY_DRIVE_TRAINS:
             electric_data = state.get("electricChargingState", {})
@@ -101,18 +110,29 @@ class FuelAndBattery(VehicleDataBase):  # pylint:disable=too-many-instance-attri
                     cls._parse_electric_data(
                         electric_data,
                         vehicle_data["fetched_at"],
-                        vehicle_data["is_metric"],
                         state.get("chargingProfile", {}).get("reductionOfChargeCurrent"),
                     ),
                 )
 
         if drivetrain in set(COMBUSTION_ENGINE_DRIVE_TRAINS).intersection(HV_BATTERY_DRIVE_TRAINS):
+            # for hybrid vehicles the remaining_range_fuel returned by the API seems to be the total remaining range
+            # to calculate the correct remaining range on fuel, we have to subtract the remaining electric range
+            # this does not seem to apply for the old I3 with range extender
+
             fuel: ValueWithUnit = retval.get("remaining_range_fuel", ValueWithUnit(None, None))
             electric: ValueWithUnit = retval.get("remaining_range_electric", ValueWithUnit(None, None))
-            retval["remaining_range_total"] = ValueWithUnit(
-                (fuel.value or 0) + (electric.value or 0),
-                fuel.unit or electric.unit,
-            )
+
+            if drivetrain == DriveTrainType.ELECTRIC_WITH_RANGE_EXTENDER:
+                retval["remaining_range_total"] = ValueWithUnit(
+                    (fuel.value or 0) + (electric.value or 0),
+                    fuel.unit or electric.unit,
+                )
+            else:
+                retval["remaining_range_total"] = retval["remaining_range_fuel"]
+                retval["remaining_range_fuel"] = ValueWithUnit(
+                    (fuel.value or 0) - (electric.value or 0),
+                    fuel.unit or electric.unit,
+                )
         elif drivetrain in COMBUSTION_ENGINE_DRIVE_TRAINS and "remaining_range_fuel" in retval:
             retval["remaining_range_total"] = retval["remaining_range_fuel"]
         elif drivetrain in HV_BATTERY_DRIVE_TRAINS and "remaining_range_electric" in retval:
@@ -121,20 +141,20 @@ class FuelAndBattery(VehicleDataBase):  # pylint:disable=too-many-instance-attri
         return retval
 
     @staticmethod
-    def _parse_fuel_data(fuel_data: Dict, is_metric: bool) -> Dict:
+    def _parse_fuel_data(fuel_data: Dict) -> Dict:
         """Parse fuel data."""
         retval = {}
         if "remainingFuelLiters" in fuel_data:
-            retval["remaining_fuel"] = ValueWithUnit(fuel_data["remainingFuelLiters"], "L" if is_metric else "gal")
+            retval["remaining_fuel"] = ValueWithUnit(fuel_data["remainingFuelLiters"], "L")
         if "remainingFuelPercent" in fuel_data:
             retval["remaining_fuel_percent"] = fuel_data["remainingFuelPercent"]
         if "range" in fuel_data:
-            retval["remaining_range_fuel"] = ValueWithUnit(fuel_data["range"], "km" if is_metric else "mi")
+            retval["remaining_range_fuel"] = ValueWithUnit(fuel_data["range"], "km")
         return retval
 
     @staticmethod
     def _parse_electric_data(
-        electric_data: Dict, fetched_at: datetime.datetime, is_metric: bool, charging_window: Optional[Dict] = None
+        electric_data: Dict, fetched_at: datetime.datetime, charging_window: Optional[Dict] = None
     ) -> Dict:
         """Parse electric data."""
         retval = {}
@@ -143,7 +163,7 @@ class FuelAndBattery(VehicleDataBase):  # pylint:disable=too-many-instance-attri
         if "chargingLevelPercent" in electric_data:
             retval["remaining_battery_percent"] = int(electric_data["chargingLevelPercent"])
         if "range" in electric_data:
-            retval["remaining_range_electric"] = ValueWithUnit(electric_data["range"], "km" if is_metric else "mi")
+            retval["remaining_range_electric"] = ValueWithUnit(electric_data["range"], "km")
         if "chargingStatus" in electric_data:
             retval["charging_status"] = ChargingState(
                 electric_data["chargingStatus"] if electric_data["chargingStatus"] != "INVALID" else "NOT_CHARGING"
@@ -152,6 +172,8 @@ class FuelAndBattery(VehicleDataBase):  # pylint:disable=too-many-instance-attri
             retval["charging_end_time"] = fetched_at + datetime.timedelta(
                 minutes=electric_data["remainingChargingMinutes"]
             )
+        if "chargingTarget" in electric_data:
+            retval["charging_target"] = int(electric_data["chargingTarget"])
 
         if retval["charging_status"] == ChargingState.WAITING_FOR_CHARGING and isinstance(charging_window, Dict):
             retval["charging_start_time_no_tz"] = datetime.datetime.combine(
