@@ -4,17 +4,25 @@ import logging
 from typing import TYPE_CHECKING, Dict, List, Optional, Tuple, Type
 
 from bimmer_connected.api.client import MyBMWClient
-from bimmer_connected.const import ATTR_ATTRIBUTES, ATTR_CAPABILITIES, ATTR_STATE, VEHICLE_IMAGE_URL, CarBrands
+from bimmer_connected.const import (
+    ATTR_ATTRIBUTES,
+    ATTR_CAPABILITIES,
+    ATTR_CHARGING_SETTINGS,
+    ATTR_STATE,
+    VEHICLE_IMAGE_URL,
+    CarBrands,
+)
 from bimmer_connected.models import StrEnum, ValueWithUnit
-from bimmer_connected.utils import deprecated, parse_datetime
+from bimmer_connected.utils import parse_datetime
 from bimmer_connected.vehicle.charging_profile import ChargingProfile
+from bimmer_connected.vehicle.climate import Climate
 from bimmer_connected.vehicle.const import COMBUSTION_ENGINE_DRIVE_TRAINS, HV_BATTERY_DRIVE_TRAINS, DriveTrainType
 from bimmer_connected.vehicle.doors_windows import DoorsAndWindows
 from bimmer_connected.vehicle.fuel_and_battery import FuelAndBattery
 from bimmer_connected.vehicle.location import VehicleLocation
 from bimmer_connected.vehicle.remote_services import RemoteServices
-from bimmer_connected.vehicle.reports import CheckControlMessageReport, ConditionBasedServiceReport
-from bimmer_connected.vehicle.vehicle_status import VehicleStatus
+from bimmer_connected.vehicle.reports import CheckControlMessageReport, ConditionBasedServiceReport, Headunit
+from bimmer_connected.vehicle.tires import Tires
 
 if TYPE_CHECKING:
     from bimmer_connected.account import MyBMWAccount
@@ -30,11 +38,11 @@ class VehicleViewDirection(StrEnum):
     This is used to get a rendered image of the vehicle.
     """
 
-    FRONTSIDE = "VehicleStatus"
-    FRONT = "VehicleInfo"
+    FRONTSIDE = "VehicleStatus"  # also available as AngleSideViewForty
+    FRONT = "FrontView"
     # REARSIDE = 'REARSIDE'
     # REAR = 'REAR'
-    SIDE = "ChargingHistory"
+    SIDE = "SideViewLeft"
     # DASHBOARD = 'DASHBOARD'
     # DRIVERDOOR = 'DRIVERDOOR'
     # REARBIRDSEYE = 'REARBIRDSEYE'
@@ -53,7 +61,6 @@ class LscType(StrEnum):
     UNKNOWN = "UNKNOWN"
 
 
-# pylint: disable=too-many-public-methods,too-many-instance-attributes
 class MyBMWVehicle:
     """Models state and remote services of one vehicle.
 
@@ -65,28 +72,35 @@ class MyBMWVehicle:
         self,
         account: "MyBMWAccount",
         vehicle_base: dict,
-        vehicle_state: dict,
+        vehicle_state: Optional[dict] = None,
+        charging_settings: Optional[dict] = None,
         fetched_at: Optional[datetime.datetime] = None,
     ) -> None:
-        """Initializes a MyBMWVehicle."""
+        """Initialize a MyBMWVehicle."""
         self.account = account
-        self.data = self.combine_data(account, vehicle_base, vehicle_state, fetched_at)
-        self.status = VehicleStatus(self)
+        self.data = self.combine_data(account, vehicle_base, vehicle_state, charging_settings, fetched_at)
         self.remote_services = RemoteServices(self)
         self.fuel_and_battery: FuelAndBattery = FuelAndBattery(account_timezone=account.timezone)
         self.vehicle_location: VehicleLocation = VehicleLocation(account_region=account.region)
         self.doors_and_windows: DoorsAndWindows = DoorsAndWindows()
         self.condition_based_services: ConditionBasedServiceReport = ConditionBasedServiceReport()
+        self.headunit: Headunit = Headunit()
         self.check_control_messages: CheckControlMessageReport = CheckControlMessageReport()
+        self.climate: Climate = Climate(account_timezone=account.timezone)
         self.charging_profile: Optional[ChargingProfile] = None
+        self.tires: Optional[Tires] = None
 
-        self.update_state(vehicle_base, vehicle_state, fetched_at)
+        self.update_state(vehicle_base, vehicle_state, charging_settings, fetched_at)
 
     def update_state(
-        self, vehicle_base: dict, vehicle_state: dict, fetched_at: Optional[datetime.datetime] = None
+        self,
+        vehicle_base: dict,
+        vehicle_state: Optional[dict] = None,
+        charging_settings: Optional[dict] = None,
+        fetched_at: Optional[datetime.datetime] = None,
     ) -> None:
         """Update the state of a vehicle."""
-        vehicle_data = self.combine_data(self.account, vehicle_base, vehicle_state or {}, fetched_at)
+        vehicle_data = self.combine_data(self.account, vehicle_base, vehicle_state, charging_settings, fetched_at)
         self.data = vehicle_data
 
         update_entities: List[Tuple[Type["VehicleDataBase"], str]] = [
@@ -95,24 +109,34 @@ class MyBMWVehicle:
             (DoorsAndWindows, "doors_and_windows"),
             (ConditionBasedServiceReport, "condition_based_services"),
             (CheckControlMessageReport, "check_control_messages"),
+            (Headunit, "headunit"),
+            (Climate, "climate"),
             (ChargingProfile, "charging_profile"),
+            (Tires, "tires"),
         ]
         for cls, vehicle_attribute in update_entities:
-            if getattr(self, vehicle_attribute) is None:
-                setattr(self, vehicle_attribute, cls.from_vehicle_data(vehicle_data))
-            else:
-                curr_attr: "VehicleDataBase" = getattr(self, vehicle_attribute)
-                curr_attr.update_from_vehicle_data(vehicle_data)
+            try:
+                if getattr(self, vehicle_attribute) is None:
+                    setattr(self, vehicle_attribute, cls.from_vehicle_data(vehicle_data))
+                else:
+                    curr_attr: "VehicleDataBase" = getattr(self, vehicle_attribute)
+                    curr_attr.update_from_vehicle_data(vehicle_data)
+            except (KeyError, TypeError) as ex:
+                _LOGGER.warning("Unable to update %s - (%s) %s", vehicle_attribute, type(ex).__name__, ex)
 
     @staticmethod
     def combine_data(
-        account: "MyBMWAccount", vehicle_base: dict, vehicle_state: dict, fetched_at: Optional[datetime.datetime] = None
+        account: "MyBMWAccount",
+        vehicle_base: dict,
+        vehicle_state: Optional[dict],
+        charging_settings: Optional[dict],
+        fetched_at: Optional[datetime.datetime] = None,
     ) -> Dict:
         """Combine API responses and additional information to a single dictionary."""
         return {
             **vehicle_base,
-            **vehicle_state,
-            "is_metric": account.config.use_metric_units,
+            **(vehicle_state or {}),
+            ATTR_CHARGING_SETTINGS: charging_settings or {},
             "fetched_at": fetched_at or datetime.datetime.now(datetime.timezone.utc),
         }
 
@@ -143,7 +167,7 @@ class MyBMWVehicle:
     @property
     def mileage(self) -> ValueWithUnit:
         """Get the mileage of the vehicle."""
-        return ValueWithUnit(self.data[ATTR_STATE].get("currentMileage", 0), "km" if self.data["is_metric"] else "mi")
+        return ValueWithUnit(self.data[ATTR_STATE].get("currentMileage", 0), "km")
 
     @property
     def timestamp(self) -> Optional[datetime.datetime]:
@@ -176,12 +200,13 @@ class MyBMWVehicle:
     def has_combustion_drivetrain(self) -> bool:
         """Return True if vehicle is equipped with an internal combustion engine.
 
-        In this case we can get the state of the gas tank."""
+        In this case we can get the state of the gas tank.
+        """
         return self.drive_train in COMBUSTION_ENGINE_DRIVE_TRAINS
 
     @property
     def is_charging_plan_supported(self) -> bool:
-        """Return True if charging control (weekly planner) is available."""
+        """Return True if charging profile is available and can be set via API."""
         return self.data[ATTR_CAPABILITIES].get("isChargingPlanSupported", False)
 
     @property
@@ -191,7 +216,9 @@ class MyBMWVehicle:
 
     @property
     def is_vehicle_active(self) -> bool:
-        """Check if the vehicle is active/moving.
+        """Deprecated, always returns False.
+
+        Check if the vehicle is active/moving.
 
         If the vehicle was active/moving at the time of the last status update, current position is not available.
         """
@@ -211,6 +238,61 @@ class MyBMWVehicle:
         """Return True if LastStateCall is enabled (vehicle automatically updates API)."""
         return self.data[ATTR_STATE]["isLscSupported"]
 
+    @property
+    def is_remote_set_target_soc_enabled(self) -> bool:
+        """Return True if Target SoC can be set via the API."""
+        return self.data[ATTR_CAPABILITIES].get("isChargingTargetSocEnabled", False)
+
+    @property
+    def is_remote_set_ac_limit_enabled(self) -> bool:
+        """Return True if AC limit can be set via the API."""
+        return self.data[ATTR_CAPABILITIES].get("isChargingPowerLimitEnabled", False)
+
+    @property
+    def is_remote_sendpoi_enabled(self) -> bool:
+        """Return True if POIs can be set via the API."""
+        return self.data[ATTR_CAPABILITIES].get("sendPoi", False)
+
+    @property
+    def is_remote_horn_enabled(self) -> bool:
+        """Return True if the horn can be activated via the API."""
+        return self.data[ATTR_CAPABILITIES].get("horn", False)
+
+    @property
+    def is_remote_lights_enabled(self) -> bool:
+        """Return True if the lights can be activated via the API."""
+        return self.data[ATTR_CAPABILITIES].get("lights", False)
+
+    @property
+    def is_remote_lock_enabled(self) -> bool:
+        """Return True if vehicle can be locked via the API."""
+        return self.data[ATTR_CAPABILITIES].get("lock", False)
+
+    @property
+    def is_remote_unlock_enabled(self) -> bool:
+        """Return True if POIs can be unlocked via the API."""
+        return self.data[ATTR_CAPABILITIES].get("unlock", False)
+
+    @property
+    def is_remote_climate_start_enabled(self) -> bool:
+        """Return True if AC/ventilation can be started via the API."""
+        return self.data[ATTR_CAPABILITIES].get("climateNow", False)
+
+    @property
+    def is_remote_climate_stop_enabled(self) -> bool:
+        """Return True if AC/ventilation can be stopped via the API."""
+        return "climateControlState" in self.data[ATTR_STATE]
+
+    @property
+    def is_remote_charge_start_enabled(self) -> bool:
+        """Return True if charging can be started via the API."""
+        return "START" in self.data[ATTR_CAPABILITIES].get("remoteChargingCommands", {}).get("chargingControl", [])
+
+    @property
+    def is_remote_charge_stop_enabled(self) -> bool:
+        """Return True if charging can be stop via the API."""
+        return "STOP" in self.data[ATTR_CAPABILITIES].get("remoteChargingCommands", {}).get("chargingControl", [])
+
     @property
     def drive_train_attributes(self) -> List[str]:
         """Get list of attributes available for the drive train of the vehicle.
@@ -224,6 +306,7 @@ class MyBMWVehicle:
         if self.has_electric_drivetrain:
             result += [
                 "charging_time_remaining",
+                "charging_start_time",
                 "charging_end_time",
                 "charging_time_label",
                 "charging_status",
@@ -231,6 +314,11 @@ class MyBMWVehicle:
                 "remaining_battery_percent",
                 "remaining_range_electric",
                 "last_charging_end_result",
+                "ac_current_limit",
+                "charging_target",
+                "charging_mode",
+                "charging_preferences",
+                "is_pre_entry_climatization_enabled",
             ]
         if self.has_combustion_drivetrain:
             result += ["remaining_fuel", "remaining_range_fuel", "remaining_fuel_percent"]
@@ -271,47 +359,3 @@ class MyBMWVehicle:
         async with MyBMWClient(self.account.config, brand=self.brand) as client:
             response = await client.get(url, headers={"accept": "image/png"})
         return response.content
-
-    # # # # # # # # # # # # # # #
-    # Deprecated
-    # # # # # # # # # # # # # # #
-
-    @property  # type: ignore[misc]
-    @deprecated("vehicle.has_electric_drivetrain")
-    def has_hv_battery(self) -> bool:
-        # pylint:disable=missing-function-docstring
-        return self.has_electric_drivetrain
-
-    @property  # type: ignore[misc]
-    @deprecated()
-    def has_range_extender(self) -> bool:
-        # pylint:disable=missing-function-docstring
-        return False
-
-    @property  # type: ignore[misc]
-    @deprecated("vehicle.has_combustion_drivetrain")
-    def has_internal_combustion_engine(self) -> bool:
-        # pylint:disable=missing-function-docstring
-        return self.has_combustion_drivetrain
-
-    @property  # type: ignore[misc]
-    @deprecated("vehicle.is_charging_plan_supported")
-    def has_weekly_planner_service(self) -> bool:
-        # pylint:disable=missing-function-docstring
-        return self.is_charging_plan_supported
-
-
-@deprecated("MyBMWVehicle")
-class ConnectedDriveVehicle(MyBMWVehicle):
-    """Deprecated class name for compatibility."""
-
-    def __init__(self, account: "MyBMWAccount", vehicle_dict: dict) -> None:
-        """Initializes a ConnectedDriveVehicle (deprecated)."""
-        super().__init__(account, vehicle_dict, {})
-
-    def update_state(
-        self, vehicle_base: dict, vehicle_state: Optional[dict] = None, fetched_at: Optional[datetime.datetime] = None
-    ) -> None:
-        """Update the state of a vehicle."""
-
-        super().update_state(vehicle_base, vehicle_state or {}, fetched_at)
